#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <phonon/audiooutput.h>
#include <phonon/mediaobject.h>
#include <QShortcut>
#include "playlistmodel.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void about(void);
    void help(void);
    void onesec(void);
    void deletePlaylistItems();
    void DoubleclickPlaylist(QModelIndex idx);
    void metaDataChanged(void);
    void tick(qint64 time);
    void enqueueNext(void);
    void prefinishMark(void);
    void loopStateChanged(int state);
    void loop(void);
    void sourceChanged(void);
    void totalTimeChanged(qint64 totalTime);
    void stateChanged(Phonon::State newstate, Phonon::State oldstate);
    void dobreaktimer(void);
    void doloop(void);
    void dosettings(void);
    void doshortcuts(void);
    void playpause(void);
    void back(void);
    void forward(void);
    void zero(void);
    void rewind(void);
    void tiptimer(void);
    void next(void);
    void prev(void);
    void volup(void);
    void voldown(void);
    void undo(void);
    void redo(void);
    void playlistChanged(void);
    void saveAs(void);
    void save(void);
    void closefile(void);
    void open(void);
    void dragEnterEvent(QDragEnterEvent *event); // for drag-drop
    void playlistDropped(QString filename);

protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent *event); // this lets me "trap" any window close event

private:
    Ui::MainWindow *ui;
    PlaylistModel *playlistModel;
    Phonon::MediaObject *mediaObject;
    Phonon::AudioOutput *audioOutput;
    Phonon::MediaSource source;
    int nexttrack;
    qint64 Loopstart,Loopend;
    QString Soundfile; // sound file for timer
    int PlayUsing; // 0=QSound::play, 1=custom Command
    QString Command; // custom command for playing tip timer file
    int Tiptime; // timer interval (minutes)
    int Backtime; // auto backup time (seconds)
    int Skiptime; // skip forward/back time (seconds)
    int Volumestep; // volume step (parts per hundred)
    struct position {
        int track;
        qint64 position;
    };
    QList<struct position> undolist; // undo history buffer
    int undoloc; // current position in undolist
    // current zero position is undolist[undoloc];
    qint64 queuedPosition; // position to seek to in new file (after sourceChanged), negative is offset from end of file
    bool queuedFlag; // flag that a queuedPosition seek is pending
    bool loopedFlag; // flag that file looped and prefinish mark need to be set again
    // data for the customizable shortcut keys
    QString Openfile;
    struct shortcut {
          QString Key;
          QString Action;
          QShortcut *ptrShortcut;
    };
    QList<struct shortcut> shortcutlist;
    QMap<QString, const char *> shortcutSlots;
    bool TimerRunning;
    bool TimerMode; // true for break timer mode
    int Timer;
    bool PlaylistChanged; // flag that playlist has been altered since opening/saving it

    void setCurrentPath(void);
    void readSettings(void);
    void emptyUndo(void);
    int zeroTrack();
    qint64 zeroPosition();
    void markZero(struct position here);
    void activateShortcuts(void);
    void deactivateShortcuts(void);
    void saveFile(QString filename);
    void setTitle(void);
    void saveAsk(void);
    void openPlaylist(QString filename);
    void dropEvent(QDropEvent *event); // for drag-drop
    QStringList readFileTypes(void);
};

#endif // MAINWINDOW_H
